from __future__ import annotations

from django_spire.auth.mfa.models import MfaCode
from django_spire.auth.mfa.utils import get_or_generate_user_mfa_code
from django_spire.auth.user.tests.factories import create_user
from django_spire.core.tests.test_cases import BaseTestCase


class GetOrGenerateUserMfaCodeTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.user = create_user(username='testuser')

    def test_generates_new_code_when_none_exists(self) -> None:
        mfa_code = get_or_generate_user_mfa_code(self.user)
        assert mfa_code is not None
        assert mfa_code.user == self.user
        assert mfa_code.is_valid()

    def test_returns_existing_valid_code(self) -> None:
        existing_code = MfaCode.generate_code(self.user)
        result = get_or_generate_user_mfa_code(self.user)
        assert result.pk == existing_code.pk

    def test_generates_new_code_when_existing_expired(self) -> None:
        existing_code = MfaCode.generate_code(self.user)
        existing_code.set_expired()

        new_code = get_or_generate_user_mfa_code(self.user)
        assert new_code.pk != existing_code.pk
        assert new_code.is_valid()

    def test_returns_same_code_on_multiple_calls(self) -> None:
        code1 = get_or_generate_user_mfa_code(self.user)
        code2 = get_or_generate_user_mfa_code(self.user)
        assert code1.pk == code2.pk

    def test_different_users_get_different_codes(self) -> None:
        user2 = create_user(username='testuser2')
        code1 = get_or_generate_user_mfa_code(self.user)
        code2 = get_or_generate_user_mfa_code(user2)
        assert code1.pk != code2.pk
        assert code1.user != code2.user

    def test_generated_code_is_persisted(self) -> None:
        mfa_code = get_or_generate_user_mfa_code(self.user)
        assert MfaCode.objects.filter(pk=mfa_code.pk).exists()

    def test_generates_valid_code_after_expiration(self) -> None:
        code1 = get_or_generate_user_mfa_code(self.user)
        code1.set_expired()
        code2 = get_or_generate_user_mfa_code(self.user)
        assert code2.is_valid()

    def test_code_belongs_to_correct_user(self) -> None:
        mfa_code = get_or_generate_user_mfa_code(self.user)
        assert mfa_code.user.pk == self.user.pk
        assert mfa_code.user.username == self.user.username

    def test_returns_mfa_code_instance(self) -> None:
        mfa_code = get_or_generate_user_mfa_code(self.user)
        assert isinstance(mfa_code, MfaCode)

    def test_code_has_valid_expiration(self) -> None:
        mfa_code = get_or_generate_user_mfa_code(self.user)
        assert mfa_code.expiration_datetime is not None

    def test_code_has_six_digits(self) -> None:
        mfa_code = get_or_generate_user_mfa_code(self.user)
        assert len(str(mfa_code.code)) <= 6
        assert int(mfa_code.code) <= 999999

    def test_code_is_numeric(self) -> None:
        mfa_code = get_or_generate_user_mfa_code(self.user)
        assert str(mfa_code.code).isdigit()

    def test_multiple_expired_codes_generates_new(self) -> None:
        code1 = get_or_generate_user_mfa_code(self.user)
        code1.set_expired()
        code2 = get_or_generate_user_mfa_code(self.user)
        code2.set_expired()
        code3 = get_or_generate_user_mfa_code(self.user)
        assert code3.is_valid()
        assert code3.pk != code1.pk
        assert code3.pk != code2.pk

    def test_does_not_affect_other_users_codes(self) -> None:
        user2 = create_user(username='testuser2')
        code1 = get_or_generate_user_mfa_code(self.user)
        code2 = get_or_generate_user_mfa_code(user2)
        code1.set_expired()
        result = get_or_generate_user_mfa_code(user2)
        assert result.pk == code2.pk

    def test_new_code_after_delete(self) -> None:
        code1 = get_or_generate_user_mfa_code(self.user)
        code1_pk = code1.pk
        code1.delete()
        code2 = get_or_generate_user_mfa_code(self.user)
        assert code2.pk != code1_pk

    def test_code_user_relationship_intact(self) -> None:
        mfa_code = get_or_generate_user_mfa_code(self.user)
        assert mfa_code.user == self.user
        assert mfa_code in self.user.mfa_codes.all()



# Generated by Django 5.2.10 on 2026-01-08 20:41

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ReportRun',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('report_key_stack', models.TextField()),
                ('datetime', models.DateTimeField(default=django.utils.timezone.now)),
            ],
            options={
                'verbose_name': 'Report Run',
                'verbose_name_plural': 'Report Runs',
                'db_table': 'django_spire_metric_report_run',
            },
        ),
    ]



from __future__ import annotations

from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase


class ReportPageUrlsTests(BaseTestCase):
    def setUp(self):
        super().setUp()

    def test_report_view_url_path(self):
        response = self.client.get(
            reverse('django_spire:metric:report:page:report')
        )
        assert response.status_code == 200



from __future__ import annotations

from typing import TYPE_CHECKING

from django.contrib import admin
from django.contrib import messages

from .models import Collection, CollectionGroup

if TYPE_CHECKING:
    from django.db.models import QuerySet


@admin.register(Collection)
class CollectionAdmin(admin.ModelAdmin):
    list_display = ['id', 'name', 'parent', 'is_deleted', 'tag_count']
    list_select_related = ['parent']
    list_filter = ['is_deleted', 'is_active']
    search_fields = ['id', 'name', 'description', 'parent__name']
    ordering = ['name']
    autocomplete_fields = ['parent']
    actions = ['set_tags_for_collections']

    @admin.action(description="Set Tags for Collections (Allow 5 Seconds Per)")
    def set_tags_for_collections(self, request, queryset: QuerySet[Collection]):
        processed = 0
        for collection in queryset:
            collection.services.tag.process_and_set_tags()
            processed += 1

        messages.success(request, f'Successfully processed {processed} collections.')

    def tag_count(self, collection: Collection):
        return collection.tags.count()

    tag_count.short_description = 'Tags'


@admin.register(CollectionGroup)
class CollectionGroupAdmin(admin.ModelAdmin):
    list_display = ['id', 'collection', 'auth_group']
    list_select_related = ['collection', 'auth_group']
    search_fields = ['id', 'collection__name', 'auth_group__name']





from __future__ import annotations

from pydantic import model_validator, BaseModel

from django_spire.knowledge.entry.version.block.constants import SPACES_PER_INDENT
from django_spire.knowledge.entry.version.block.data.data import BaseEditorJsBlockData
from django_spire.knowledge.entry.version.block.data.list.meta import ChecklistItemMeta, OrderedListItemMeta
from django_spire.knowledge.entry.version.block.data.list.choices import ListEditorBlockDataStyle


class ListEditorBlockData(BaseEditorJsBlockData):
    style: ListEditorBlockDataStyle | str
    meta: ChecklistItemMeta | OrderedListItemMeta | None = None
    items: list[ListItemEditorBlockData]

    def render_to_text(self) -> str:
        render_string = ''
        for i, item in enumerate(self.items):
            render_string += item.render_to_text(self.style, 0, i)

        return render_string

    @model_validator(mode='before')
    @classmethod
    def validate_meta(cls, values: dict) -> dict:
        values['style'] = ListEditorBlockDataStyle(values.get('style'))

        if values['style'] == ListEditorBlockDataStyle.ORDERED and 'meta' in values:
            values['meta'] = OrderedListItemMeta(**values['meta'])

        for item in values['items']:
            item = ListItemEditorBlockData.style_aware_create_from_dict(item, values['style'])

        return values


class ListItemEditorBlockData(BaseModel):
    content: str
    meta: ChecklistItemMeta | OrderedListItemMeta | dict | None = None
    items: list[ListItemEditorBlockData] | None = []

    def get_prefix(
        self,
        style: ListEditorBlockDataStyle,
        indent_level: int,
        index = None
    ):
        prefix = ' ' * indent_level * SPACES_PER_INDENT

        if style == ListEditorBlockDataStyle.ORDERED:
            index = index or 0
            start = self.meta.start or 1
            prefix += f'{start + index}.'
        elif style == ListEditorBlockDataStyle.CHECKLIST:
            prefix += f'[{"X" if self.meta.checked else " "}]'
        else:
            prefix += '-'

        return prefix

    def render_to_text(
        self,
        style: ListEditorBlockDataStyle,
        indent_level: int,
        index: int
    ) -> str:
        from django_spire.knowledge.entry.version.converters.markdown_converter import MarkdownConverter

        prefix = self.get_prefix(style, indent_level, index)
        parsed_content = MarkdownConverter.html_to_markdown(self.content)
        render_string = f'{prefix} {parsed_content}\n'

        for i, item in enumerate(self.items):
            render_string += item.render_to_text(style, indent_level + 1, i)

        return render_string

    @classmethod
    def style_aware_create_from_dict(
        cls,
        values: dict,
        style: ListEditorBlockDataStyle
    ) -> ListItemEditorBlockData:
        from django_spire.knowledge.entry.version.block.data.list.maps import \
            LIST_BLOCK_DATA_META_MAP

        if 'meta' in values:
            meta_type = LIST_BLOCK_DATA_META_MAP[style]

            if isinstance(values['meta'], dict):
                values['meta'] = meta_type(**values['meta']) if meta_type else None

        for item in values['items']:
            item = ListItemEditorBlockData.style_aware_create_from_dict(item, style)

        return ListItemEditorBlockData(**values)



from __future__ import annotations

from typing import TYPE_CHECKING

from django_spire.contrib.seeding.field.base import BaseFieldSeeder
from django_spire.contrib.seeding.field.enums import FieldSeederTypesEnum

if TYPE_CHECKING:
    from typing import Any


class CallableFieldSeeder(BaseFieldSeeder):
    keyword = FieldSeederTypesEnum.CALLABLE

    def seed(self, manager: Any = None, count: int = 1) -> list[dict]:
        return [
            {field_name: func[1]() for field_name, func in self.seeder_fields.items()}
            for _ in range(count)
        ]



from __future__ import annotations

from django.utils.timezone import now, timedelta

from django_spire.auth.user.tests.factories import create_user
from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.notification.choices import (
    NotificationPriorityChoices,
    NotificationStatusChoices,
    NotificationTypeChoices,
)
from django_spire.notification.models import Notification
from django_spire.notification.tests.factories import create_test_notification


class NotificationQuerySetTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.user = create_user(username='test_notification_user')
        self.notification = create_test_notification(user=self.user)

    def test_active_returns_non_deleted(self):
        result = Notification.objects.active()
        assert self.notification in result

    def test_active_excludes_deleted(self):
        self.notification.is_deleted = True
        self.notification.save()

        result = Notification.objects.active()
        assert self.notification not in result

    def test_by_user(self):
        other_user = create_user(username='other_user')
        other_notification = create_test_notification(user=other_user)

        result = Notification.objects.by_user(self.user)
        assert self.notification in result
        assert other_notification not in result

    def test_by_users(self):
        other_user = create_user(username='other_user')
        other_notification = create_test_notification(user=other_user)

        result = Notification.objects.by_users([self.user, other_user])
        assert self.notification in result
        assert other_notification in result

    def test_app_notifications(self):
        self.notification.type = NotificationTypeChoices.APP
        self.notification.save()

        email_notification = create_test_notification(
            user=self.user,
            type=NotificationTypeChoices.EMAIL
        )

        result = Notification.objects.app_notifications()
        assert self.notification in result
        assert email_notification not in result

    def test_email_notifications(self):
        self.notification.type = NotificationTypeChoices.EMAIL
        self.notification.save()

        app_notification = create_test_notification(
            user=self.user,
            type=NotificationTypeChoices.APP
        )

        result = Notification.objects.email_notifications()
        assert self.notification in result
        assert app_notification not in result

    def test_sms_notifications(self):
        self.notification.type = NotificationTypeChoices.SMS
        self.notification.save()

        result = Notification.objects.sms_notifications()
        assert self.notification in result

    def test_pending(self):
        self.notification.status = NotificationStatusChoices.PENDING
        self.notification.save()

        sent_notification = create_test_notification(
            user=self.user,
            status=NotificationStatusChoices.SENT
        )

        result = Notification.objects.pending()
        assert self.notification in result
        assert sent_notification not in result

    def test_processing(self):
        self.notification.status = NotificationStatusChoices.PROCESSING
        self.notification.save()

        result = Notification.objects.processing()
        assert self.notification in result

    def test_sent(self):
        self.notification.status = NotificationStatusChoices.SENT
        self.notification.save()

        result = Notification.objects.sent()
        assert self.notification in result

    def test_errored(self):
        self.notification.status = NotificationStatusChoices.ERRORED
        self.notification.save()

        result = Notification.objects.errored()
        assert self.notification in result

    def test_failed(self):
        self.notification.status = NotificationStatusChoices.FAILED
        self.notification.save()

        result = Notification.objects.failed()
        assert self.notification in result

    def test_unsent(self):
        pending = create_test_notification(
            user=self.user,
            status=NotificationStatusChoices.PENDING
        )
        processing = create_test_notification(
            user=self.user,
            status=NotificationStatusChoices.PROCESSING
        )
        sent = create_test_notification(
            user=self.user,
            status=NotificationStatusChoices.SENT
        )

        result = Notification.objects.unsent()
        assert pending in result
        assert processing in result
        assert sent not in result

    def test_ready_to_send(self):
        past_notification = create_test_notification(
            user=self.user,
            status=NotificationStatusChoices.PENDING,
            publish_datetime=now() - timedelta(hours=1)
        )
        future_notification = create_test_notification(
            user=self.user,
            status=NotificationStatusChoices.PENDING,
            publish_datetime=now() + timedelta(hours=1)
        )

        result = Notification.objects.ready_to_send()
        assert past_notification in result
        assert future_notification not in result

    def test_high_priority(self):
        self.notification.priority = NotificationPriorityChoices.HIGH
        self.notification.save()

        low_priority = create_test_notification(
            user=self.user,
            priority=NotificationPriorityChoices.LOW
        )

        result = Notification.objects.high_priority()
        assert self.notification in result
        assert low_priority not in result

    def test_medium_priority(self):
        self.notification.priority = NotificationPriorityChoices.MEDIUM
        self.notification.save()

        result = Notification.objects.medium_priority()
        assert self.notification in result

    def test_low_priority(self):
        self.notification.priority = NotificationPriorityChoices.LOW
        self.notification.save()

        result = Notification.objects.low_priority()
        assert self.notification in result



from __future__ import annotations

from typing import TYPE_CHECKING

from django_spire.notification.app.processor import AppNotificationProcessor
from django_spire.notification.email.processor import EmailNotificationProcessor
from django_spire.notification.processors.notification import NotificationProcessor
from django_spire.notification.sms.processor import SMSNotificationProcessor

if TYPE_CHECKING:
    from django_spire.notification.models import Notification


class NotificationManager:
    @staticmethod
    def process_errored_notifications():
        NotificationProcessor().process_errored()

    @staticmethod
    def process_errored_app_notifications():
        AppNotificationProcessor().process_errored()

    @staticmethod
    def process_errored_email_notifications():
        EmailNotificationProcessor().process_errored()

    @staticmethod
    def process_errored_sms_notifications():
        SMSNotificationProcessor().process_errored()

    @staticmethod
    def process_notification(notification: Notification):
        NotificationProcessor().process(notification)

    @staticmethod
    def process_notifications(notifications: list[Notification]):
        NotificationProcessor().process_list(notifications)

    @staticmethod
    def process_ready_app_notifications():
        AppNotificationProcessor().process_ready()

    @staticmethod
    def process_ready_email_notifications():
        EmailNotificationProcessor().process_ready()

    @staticmethod
    def process_ready_sms_notifications():
        SMSNotificationProcessor().process_ready()

    @staticmethod
    def process_ready_notifications():
        NotificationProcessor().process_ready()



from __future__ import annotations

import json

import django_glue as dg

from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import reverse

from django_glue.utils import serialize_to_json

from django_spire.auth.group.models import AuthGroup
from django_spire.auth.permissions.decorators import permission_required
from django_spire.auth.user import forms
from django_spire.auth.user.models import AuthUser
from django_spire.auth.user.tools import add_user_to_all_user_group
from django_spire.contrib import Breadcrumbs
from django_spire.contrib.form.utils import show_form_errors
from django_spire.contrib.generic_views import portal_views
from django_spire.history.activity.utils import add_form_activity
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.template.response import TemplateResponse


@permission_required('django_spire_auth_user.add_authuser')
def register_form_view(request: WSGIRequest) -> TemplateResponse:
    portal_user = AuthUser()
    dg.glue_model_object(request, 'portal_user', portal_user, 'view')

    if request.method == 'POST':
        user_form = forms.RegisterUserForm(request.POST, instance=portal_user)

        if user_form.is_valid():
            user = user_form.save()
            add_user_to_all_user_group(user)

            add_form_activity(user, 0, request.user)

            return HttpResponseRedirect(reverse('django_spire:auth:user:page:list'))

        show_form_errors(request, user_form)
    else:
        user_form = forms.RegisterUserForm(instance=portal_user)

    context_data = {
        # TODO: Function that takes in all of the forms and dumps the data here?
        'user_form_data': json.dumps(user_form.data, cls=DjangoJSONEncoder),
    }

    crumbs = Breadcrumbs()
    crumbs.add_breadcrumb(name='Users', href=reverse('django_spire:auth:user:page:list'))
    crumbs.add_breadcrumb(name='Register New User')

    return portal_views.template_view(
        request,
        context_data=context_data,
        page_title='Register',
        page_description='New User',
        breadcrumbs=crumbs,
        template='django_spire/auth/user/page/register_form_page.html'
    )


@permission_required('django_spire_auth_user.change_authuser')
def form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    portal_user = get_object_or_404(AuthUser, pk=pk)
    dg.glue_model_object(request, 'portal_user', portal_user, 'view')

    if request.method == 'POST':
        form = forms.UserForm(request.POST, instance=portal_user)

        if form.is_valid():
            portal_user = form.save()
            add_form_activity(portal_user, pk, request.user)

            return HttpResponseRedirect(
                reverse(
                    'django_spire:auth:user:page:detail',
                    kwargs={'pk': pk}
                )
            )
    else:
        form = forms.UserForm(instance=portal_user)

    context_data = {
        'portal_user': portal_user,
        'initial_data': serialize_to_json(form.data)
    }

    return portal_views.model_form_view(
        request,
        form=form,
        obj=portal_user,
        context_data=context_data,
        template='django_spire/auth/user/page/form_page.html'
    )


@permission_required('django_spire_auth_group.change_authgroup')
def group_form_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    user = get_object_or_404(AuthUser, pk=pk)
    dg.glue_query_set(request, 'group_choices', AuthGroup.objects.all())
    selected_group_ids = [group.pk for group in user.groups.all()]

    if request.method == 'POST':
        form = forms.UserGroupForm(request.POST)

        if form.is_valid():
            user.groups.set(form.cleaned_data['group_list'])
            return HttpResponseRedirect(reverse('django_spire:auth:user:page:detail', kwargs={'pk': pk}))

        show_form_errors(request, form)

    form = forms.UserGroupForm()

    context_data = {
        'user': user,
        'selected_group_ids': selected_group_ids
    }

    return portal_views.form_view(
        request,
        obj=user,
        context_data=context_data,
        form=forms.UserGroupForm(),
        template='django_spire/auth/user/page/group_form_page.html'
    )



from __future__ import annotations

from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.notification.app.tests.factories import create_test_app_notification


class AppNotificationPageViewsTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.app_notification = create_test_app_notification(user=self.super_user)

    def test_app_notification_list_view_status_code(self):
        response = self.client.get(
            reverse('django_spire:notification:app:page:list')
        )

        assert response.status_code == 200



from django.urls import path, include


app_name = 'auth'

urlpatterns = [
    path('', include('django_spire.auth.urls.admin_urls', namespace='admin')),
    path('redirect/', include('django_spire.auth.urls.redirect_urls', namespace='redirect')),
    path('mfa/', include('django_spire.auth.mfa.urls', namespace='mfa')),
    path('user/', include('django_spire.auth.user.urls', namespace='user')),
    path('group/', include('django_spire.auth.group.urls', namespace='group')),
]



from __future__ import annotations

from typing import TYPE_CHECKING

from dandy.recorder import recorder_to_html_file

from django_spire.ai.chat.intelligence.workflows.chat_workflow import chat_workflow
from django_spire.ai.sms.intel import SmsIntel

if TYPE_CHECKING:
    from dandy.llm.request.message import MessageHistory
    from django.core.handlers.wsgi import WSGIRequest


@recorder_to_html_file('spire_ai_sms_conversation_workflow')
def sms_conversation_workflow(
    request: WSGIRequest, user_input: str, message_history: MessageHistory | None = None, actor: str | None = None
) -> SmsIntel:
    message_intel = chat_workflow(
        request,
        user_input=user_input,
        message_history=message_history,
    )

    return SmsIntel(
        body=message_intel.render_to_str()
    )



from __future__ import annotations

from django.apps import apps
from django.urls import include, path

from django_spire.exceptions import DjangoSpireConfigurationError


app_name = 'django_spire'

urlpatterns = [
    path(
        f'{app_config.URLPATTERNS_NAMESPACE}/',
        include(
            app_config.URLPATTERNS_INCLUDE,
            namespace=app_config.URLPATTERNS_NAMESPACE,
        )
    )
    for app_config in apps.get_app_configs()
    if hasattr(app_config, 'URLPATTERNS_INCLUDE') and hasattr(app_config, 'URLPATTERNS_NAMESPACE')
]

if not urlpatterns:
    message = 'You need to have at least one app installed to use Django Spire.'
    raise DjangoSpireConfigurationError(message)



from __future__ import annotations


def get_matching_tag_set_from_tag_sets(tag_set_a: set[str], tag_set_b: set[str]) -> set[str]:
    return tag_set_a.intersection(tag_set_b)


def get_matching_count_from_tag_sets(tag_set_a: set[str], tag_set_b: set[str]) -> int:
    return len(get_matching_tag_set_from_tag_sets(tag_set_a, tag_set_b))


def get_matching_a_percentage_from_tag_sets(tag_set_a: set[str], tag_set_b: set[str]) -> float:
    try:
        return get_matching_count_from_tag_sets(tag_set_a, tag_set_b) / len(tag_set_a)
    except ZeroDivisionError:
        return 0.0


def get_matching_b_percentage_from_tag_sets(tag_set_a: set[str], tag_set_b: set[str]) -> float:
    try:
        return get_matching_count_from_tag_sets(tag_set_a, tag_set_b) / len(tag_set_b)
    except ZeroDivisionError:
        return 0.0


def get_score_percentage_from_tag_set_weighted(tag_set_actual: set[str], tag_set_reference: set[str]) -> float:
    total_points = len(simplify_tag_set(tag_set_reference))

    multiplier = get_matching_count_from_tag_sets(tag_set_actual, tag_set_reference)

    weighted_points = sum(
        weight
        for tag, weight in simplify_and_weight_tag_set_to_dict(tag_set_reference).items()
        if tag in tag_set_actual
    )

    if weighted_points == 0 or total_points == 0:
        return 0.0

    return (weighted_points / total_points) * multiplier


def simplify_tag_set(tag_set: set[str]) -> set[str]:
    return set(simplify_tag_set_to_list(tag_set))


def simplify_tag_set_to_list(tag_set: set[str]) -> list[str]:
    simplified_tag_words = []

    for tag in tag_set:
        tag_words = tag.split('-')
        simplified_tag_words.extend(tag_words)

    return simplified_tag_words


def simplify_and_weight_tag_set_to_dict(tag_set: set[str]) -> dict[str, int]:
    simplified_and_weighted_tag_words = {}

    for tag_word in simplify_tag_set_to_list(tag_set):
        simplified_and_weighted_tag_words[tag_word] = simplified_and_weighted_tag_words.get(tag_word, 0) + 1

    return dict(
        sorted(
            simplified_and_weighted_tag_words.items(),
            key=lambda item: item[1],
            reverse=True
        )
    )
